/*
  MathJinni - A simple formular calculator
  Copyright (C) 2007  Tim Teulings

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "FunctionInput.h"

#include <Lum/Base/String.h>

#include <Lum/Button.h>
#include <Lum/Grid.h>

static FunctionInput::Prefs* prefs=new FunctionInput::Prefs();

FunctionInput::Prefs::Prefs()
 : Lum::Component::Prefs(L"FunctionInput")
{
  // no code
}

void FunctionInput::Prefs::Initialize()
{
  Lum::Component::Prefs::Initialize();

  frame=new Lum::OS::EmptyFrame();
}

FunctionInput::FunctionInput()
 : backspaceAction(new Lum::Model::Action()),
   clearAction(new Lum::Model::Action())
{
  SetPrefs(::prefs);

  AttachModel(backspaceAction);
  AttachModel(clearAction);
}

FunctionInput::~FunctionInput()
{
  UnattachModel(clearAction);
  UnattachModel(backspaceAction);
}

void FunctionInput::SetString(Lum::String *string)
{
  this->string=string;
}

void FunctionInput::SetCalculateAction(Lum::Model::Action* action)
{
  calculateAction=action;
}

Lum::Object* FunctionInput::CreateButton(const std::wstring& label, Lum::Model::Action *action) const
{
  Lum::Button *button;

  button=new Lum::Button();
  button->SetFlex(true,true);

  if (action!=NULL) {
    button->SetModel(action);
  }

  button->SetText(label);

  return button;
}

Lum::Object* FunctionInput::CreateTextButton(const std::wstring& label, const std::wstring& text)
{
  Lum::Model::ActionRef action;

  action=new Lum::Model::Action();

  AttachModel(action);

  if (text.empty()) {
    actionTextMap[action.Get()]=label;
  }
  else {
    actionTextMap[action.Get()]=text;
  }

  return CreateButton(label,action);
}

void FunctionInput::CalcSize()
{
  Lum::Grid *grid;

  grid=new Lum::Grid();
  grid->SetFlex(true,true);
  grid->SetSize(8,5);
  grid->SetEqualDimensions(true,true);
  grid->SetSpace(true,true);

  //grid->SetObject(0,0,CreateTextButton(L"abs"));
  grid->SetObject(1,0,CreateTextButton(L"e"));
  grid->SetObject(2,0,CreateTextButton(L"ln",L"ln("));
  grid->SetObject(3,0,CreateTextButton(L"lg",L"lg("));
  grid->SetObject(4,0,CreateTextButton(L"("));
  grid->SetObject(5,0,CreateTextButton(L")"));
  grid->SetObject(6,0,CreateButton(L"<-",backspaceAction));
  grid->SetObject(7,0,CreateButton(L"Clr",clearAction));

  grid->SetObject(1,1,CreateTextButton(L"^"));
  grid->SetObject(2,1,CreateTextButton(L"sqrt",L"sqrt("));
  grid->SetObject(3,1,CreateTextButton(L"abs",L"abs("));
  grid->SetObject(4,1,CreateTextButton(L"7"));
  grid->SetObject(5,1,CreateTextButton(L"8"));
  grid->SetObject(6,1,CreateTextButton(L"9"));
  grid->SetObject(7,1,CreateTextButton(L"/"));

  grid->SetObject(0,2,CreateTextButton(L"atanh",L"atanh("));
  grid->SetObject(1,2,CreateTextButton(L"sinh",L"sinh("));
  grid->SetObject(2,2,CreateTextButton(L"cosh",L"cosh"));
  grid->SetObject(3,2,CreateTextButton(L"tanh",L"tanh"));
  grid->SetObject(4,2,CreateTextButton(L"4"));
  grid->SetObject(5,2,CreateTextButton(L"5"));
  grid->SetObject(6,2,CreateTextButton(L"6"));
  grid->SetObject(7,2,CreateTextButton(L"*"));

  grid->SetObject(1,3,CreateTextButton(L"asin",L"asin("));
  grid->SetObject(2,3,CreateTextButton(L"acos",L"acos("));
  grid->SetObject(3,3,CreateTextButton(L"atan",L"atan("));
  grid->SetObject(4,3,CreateTextButton(L"1"));
  grid->SetObject(5,3,CreateTextButton(L"2"));
  grid->SetObject(6,3,CreateTextButton(L"3"));
  grid->SetObject(7,3,CreateTextButton(L"-"));

  grid->SetObject(0,4,CreateTextButton(L"pi"));
  grid->SetObject(1,4,CreateTextButton(L"sin",L"sin("));
  grid->SetObject(2,4,CreateTextButton(L"cos",L"cos("));
  grid->SetObject(3,4,CreateTextButton(L"tan",L"tan("));
  grid->SetObject(4,4,CreateTextButton(L"0"));
  grid->SetObject(5,4,CreateTextButton(Lum::Base::GetDecimalPointWString()));
  if (calculateAction.Valid()) {
    grid->SetObject(6,4,CreateButton(L"=",calculateAction));
  }
  else {
    grid->SetObject(6,4,CreateTextButton(L"x"));
  }
  grid->SetObject(7,4,CreateTextButton(L"+"));

  container=grid;

  container->SetParent(this);
  container->CalcSize();

  minWidth=container->GetOMinWidth();
  minHeight=container->GetOMinHeight();
  width=container->GetOWidth();
  height=container->GetOHeight();

  Component::CalcSize();
}

void FunctionInput::Resync(Lum::Base::Model* model, const Lum::Base::ResyncMsg& msg)
{
  if (model==backspaceAction && backspaceAction->IsFinished()) {
    string->ActionBackspace();
  }
  else if (model==clearAction && clearAction->IsFinished()) {
    string->ActionEraseAll();
  }
  else if (dynamic_cast<Lum::Model::Action*>(model)!=NULL &&
           dynamic_cast<Lum::Model::Action*>(model)->IsFinished()) {
    std::map<Lum::Model::Action*,std::wstring>::const_iterator iter;

    iter=actionTextMap.find(dynamic_cast<Lum::Model::Action*>(model));
    if (iter!=actionTextMap.end()) {
      string->ActionInsertString(iter->second);
    }
  }

  Component::Resync(model,msg);
}

